/* $Id: hellodl.c,v 1.4 2003/02/28 21:16:29 tm Exp $
 * PDFlib client: hello example in C with dynamic DLL loading
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "pdflibdl.h"

int
main(void)
{
    PDF *p;
    int font;
    PDFlib_api *PDFlib;
    
    /* load the PDFlib dynamic library  */
    if ((PDFlib = PDF_boot_dll()) == NULL)
    {
	fprintf(stderr, "Error: couldn't boot PDFlib DLL.\n");
	return(2);
    }

    /* create a new PDFlib object */
    if ((p = PDFlib->PDF_new()) == (PDF *) 0)
    {
        printf("Couldn't create PDFlib object (out of memory)!\n");
        return(2);
    }

    PDF_TRY(p) {
	/* open new PDF file */
	if (PDFlib->PDF_open_file(p, "hellodl.pdf") == -1) {
	    fprintf(stderr, "Error: cannot open PDF file hellodl.pdf.\n");
	    return(2);
	}

	PDFlib->PDF_set_info(p, "Creator", "hello.c");
	PDFlib->PDF_set_info(p, "Author", "Thomas Merz");
	PDFlib->PDF_set_info(p, "Title", "Hello, world (C DLL)!");

	PDFlib->PDF_begin_page(p, a4_width, a4_height);	/* start a new page */

	/* Change "host" encoding to "winansi" or whatever you need! */
	font = PDFlib->PDF_load_font(p, "Helvetica-Bold", 0, "host", "");

	PDFlib->PDF_setfont(p, font, 24);
	PDFlib->PDF_set_text_pos(p, 50, 700);
	PDFlib->PDF_show(p, "Hello, world!");
	PDFlib->PDF_continue_text(p, "(says C DLL)");
	PDFlib->PDF_end_page(p);			/* close page */

	PDFlib->PDF_close(p);			/* close PDF document */
    }

    PDF_CATCH(p) {
        printf("PDFlib exception occurred in hellodl sample:\n");
        printf("[%d] %s: %s\n",
	    PDFlib->PDF_get_errnum(p), PDFlib->PDF_get_apiname(p),
	    PDFlib->PDF_get_errmsg(p));
	PDFlib->PDF_delete(p);
        return(2);
    }


    PDFlib->PDF_delete(p);			/* delete the PDFlib object */

    PDF_shutdown_dll(PDFlib);			/* unload the library */

    return 0;
}
