/*---------------------------------------------------------------------------*
 |              PDFlib - A library for generating PDF on the fly             |
 +---------------------------------------------------------------------------+
 | Copyright (c) 1997-2003 Thomas Merz and PDFlib GmbH. All rights reserved. |
 +---------------------------------------------------------------------------+
 |                                                                           |
 |    This software is subject to the PDFlib license. It is NOT in the       |
 |    public domain. Extended versions and commercial licenses are           |
 |    available, please check http://www.pdflib.com.                         |
 |                                                                           |
 *---------------------------------------------------------------------------*/

/* $Id: pc_unicode.h,v 1.26 2003/03/05 21:07:18 kurt Exp $
 *
 * Unicode glyph name conversion routines
 *
 */

#ifndef PC_UNICODE_H
#define PC_UNICODE_H

#define PDC_MAX_UNICODE         (int) 0x0000FFFF

#define PDC_REPL_CHAR           (int) 0x0000FFFD

#define PDC_EURO_SIGN           (pdc_ushort) 0x20AC

/* The Unicode byte order mark (BOM) byte parts */
#define PDF_BOM0		0376		/* '\xFE' */
#define PDF_BOM1                0377            /* '\xFF' */
#define PDF_BOM2                0357            /* '\xEF' */
#define PDF_BOM3                0273            /* '\xBB' */
#define PDF_BOM4                0277            /* '\xBF' */

/*
 * check whether the string is plain C or UTF16 unicode
 * by looking for the BOM in big-endian format
 * s must not be NULL.
 */
#define pdc_is_unicode(s) \
        (((pdc_byte *)(s))[0] == PDF_BOM0 && \
         ((pdc_byte *)(s))[1] == PDF_BOM1)

/*
 * check whether the string is plain C or UTF8 unicode
 * by looking for the BOM
 * s must not be NULL.
 */
#define pdc_is_utf8_unicode(s) \
        (((pdc_byte *)(s))[0] == PDF_BOM2 && \
         ((pdc_byte *)(s))[1] == PDF_BOM3 && \
         ((pdc_byte *)(s))[2] == PDF_BOM4)

typedef struct
{
    pdc_ushort code;
    const char *glyphname;
} pdc_glyph_tab;

typedef enum
{
    conversionOK,           /* conversion successful */
    sourceExhausted,        /* partial character in source, but hit end */
    targetExhausted,        /* insuff. room in target for conversion */
    sourceIllegal           /* source sequence is illegal/malformed */
} pdc_convers_result;

typedef enum
{
    strictConversion = 0,
    lenientConversion
} pdc_convers_flags;

typedef enum
{
    onlyConv,           /* conversion only */
    noBOM,              /* result without BOM */
    withBOMSys,         /* result with BOM - determined by system */
    withBOMInp,         /* result with BOM - determined by input if possible */
    withBOMle,          /* result with BOM - little endian */
    withBOMbe           /* result with BOM - big endian */
} pdc_bom_kind;

typedef enum
{
    pdc_auto,           /* pdc_bytes or pdc_utf16 */
    pdc_auto2,          /* pdc_bytes2 or pdc_utf16 */
    pdc_bytes,          /* simple 8-bit text */
    pdc_bytes2,         /* simple 8-bit text in 2 bytes */
    pdc_utf8,           /* text in UTF-8 format */
    pdc_utf16,          /* text in UTF-16 format in machine byte order */
    pdc_utf16be,        /* text in UTF-16 format in big endian */
    pdc_utf16le         /* text in UTF-16 format in little endian */
                        /* BOMs will be ignored in the last 4 cases */
} pdc_text_format;

pdc_ushort pdc_adobe2unicode(const char *name);

const char *pdc_unicode2adobe(pdc_ushort uv);

unsigned int pdc_name2sid(const char *name);

const char *pdc_sid2name(unsigned int sid);

pdc_bool pdc_is_std_charname(const char *name);

pdc_byte *pdc_convert_string(pdc_core *pdc, int informat, int outformat,
                             const pdc_byte *instring, int inlen,
                             pdc_bom_kind bom, int *outlen);

pdc_byte *pdc_convert_text(pdc_core *pdc, pdc_text_format *o_inutf,
                           const pdc_byte *intext, int inlen, int *outlen);

pdc_byte *pdc_convert_hypertext(pdc_core *pdc, pdc_text_format inutf,
                                pdc_encodingvector *ev,
                                pdc_encodingvector *ev_pde,
                                const pdc_byte *intext, int inlen);

#endif /* PC_UNICODE_H */
